#ifndef TASK_SERVER_TASK_H
#define TASK_SERVER_TASK_H

#include <l4.h>

enum {
  TASK_CREATE = 0,
  TASK_DESTROY,
  THREAD_CREATE,
  THREAD_DESTROY,
  TRANSACTION_SEND_CREATE,
  TRANSACTION_RECEIVE,
  TRANSACTION_DESTROY,
} task_operation;

enum {
  TASK_DEATH_NOTIFICATION = 0,
} task_callback;

enum {
  ERR_GO_AWAY = 1,
  ERR_UNKNOWN_REQUEST,
  ERR_BAD_ARGS,
  ERR_PERM,
  ERR_NOMEM,
  ERR_STATE,
  ERR_NO_TASK,
  ERR_NO_TRANSACTION,
  ERR_NO_THREAD,
  ERR_TRANSACTION_INTERRUPTED,
  ERR_TRANSACTION_FAIL,
  /* An L4 operation failed */
  ERR_FAIL,
} task_error;

/* For simplicity, use fix size, reallocation could be added later.
 * Should be a power of two.*/
#define TASK_MAX 1024
#define THREAD_MAX 1024

#define TASK_CONTROL_MASK (1UL << 14)

/* Place these at some random high address */
#define KIP_LOCATION (0xC000000UL)
#define UTC_LOCATION (0xD000000UL)

void
task_init(void);

l4_word_t
task_alloc_id(void);

l4_word_t
task_reference_handle(l4_word_t id);

l4_word_t
task_control_handle(l4_word_t id);

l4_word_t
task_by_thread(l4_thread_id_t thread);

/* Checks that the handle is syntactically valid and that OWNER has
   reference or control rights. */
l4_word_t
task_by_handle(l4_word_t owner, l4_word_t handle);

/* Checks that the handle is syntactically valid and that OWNER has
   control rights. */
l4_word_t
task_by_control_handle(l4_word_t owner, l4_word_t handle);


/* Handling references */

l4_word_t
task_reference_p(l4_word_t a, l4_word_t b);

void
task_reference_add(l4_word_t a, l4_word_t b);

void
task_reference_clear(l4_word_t a, l4_word_t b);

l4_word_t
task_control_p(l4_word_t a, l4_word_t b);

void
task_control_add(l4_word_t a, l4_word_t b);

void
task_control_clear(l4_word_t a, l4_word_t b);

void
task_transaction_incref(l4_word_t id);

void
task_transaction_decref(l4_word_t id);


/* Operations */
l4_word_t
task_create(l4_word_t creator, l4_word_t thread_max, l4_word_t *handle);

l4_word_t
task_thread_create(l4_word_t id, l4_thread_id_t pager,
		   l4_thread_id_t *new_thread);

l4_word_t
task_thread_destroy(l4_word_t id, l4_thread_id_t victim);

#endif /* TASK_SERVER_TASK_H */
